/*
**
** Init.c, from MixerKernel
**
** Initialization procedures for the mixer.
**
** sam-lo_p@epita.fr
*/

/*
** ---------- mixer_initialize 
**
** returns -1 if one of the DSPs is locked, 
** -2 if one of the DSPs fails to open, or
** -3 if it fails to initialization
**
*/

#include<fcntl.h>
#include"DebugLib.h"
#include"DspDriver.h"
#include"DspLib.h"
#include"Constants.h"
#include"MixerTypes.h"
#include"DebugLib.h"

#define DEBUGME

#ifdef DEBUGME
#include<stdio.h>
#endif

int mixer_initialize(main_mixer)
     t_main_mixer* main_mixer;
{
  t_mixer_elt* mixer_elt;
  t_dsp*       dsp[200];
  int          i;

  /* Initialisation des variables */
  mixer_elt = main_mixer->first_mixer_elt;
  for (i=0;i<main_mixer->nb_voices;i++)
    {
      dsp[i] = NULL;
    }

  /* Initialisation des DSPs */
  while (mixer_elt != NULL)
    {
      i=0;
      if (mixer_elt->dsp == NULL)
	{
	  warning("Init.c: mixer_initialize: one of the t_mixer_elt "
		  "has a NULL dsp!");
	} /* fi */

      /* Initialisation du DSP */
      while ((dsp[i] != NULL) && (dsp[i] != mixer_elt->dsp) && 
	     (i<main_mixer->nb_voices))
	{
	  i++;
	} /* while */
      
      if (dsp[i] == NULL)
	{
	  dsp[i] = mixer_elt->dsp;
	  
	  if (dsp[i]->lock == TRUE_)
	    {
	      warning("Init.c: mixer_initialize: one of the DSPs is "
		      "already locked by another player. Returning value -1");
	      return(-1);
	      
	    } /* fi */
	  
	  dsp[i]->lock = TRUE_;
	  
	  /*
	  ** Initialisation du DSP en lecture/criture
	  */
	  
	  if ((mixer_elt->rec_or_play == REC) && (dsp[i]->rec_or_play == PLAY)
	      && (mixer_elt->dsp->duplex == FALSE_))
	    {
	      dsp_close(dsp[i]->file);
	      if ((dsp[i]->file = open(dsp[i]->name,O_RDONLY)) == -1 ) 
		{
		  warning("Init.c: mixer_initialize: Erreur lors de "
			  "l'ouverture du device DSP en lecture - return(-2)");
		  return (-2);
		} /* fi */

	      dsp[i]->rec_or_play = REC;

	    } /* fi */
	  
	  if ((mixer_elt->rec_or_play == PLAY) && (dsp[i]->rec_or_play == REC)
	      && (mixer_elt->dsp->duplex == FALSE_))
	    {
	      dsp_close(dsp[i]->file);
	      if ((dsp[i]->file = open(dsp[i]->name,O_WRONLY)) == -1 ) 
		{
		  warning("Init.c: mixer_initialize: Erreur lors de "
			  "l'ouverture du device DSP en criture - "
			  "return(-2)");
		  return (-2);
		} /* fi */
	      
	      dsp[i]->rec_or_play = PLAY;
	      
	    } /* fi */
	  
	  if (dsp[i]->esd_client <0)
	    if (dsp_init(dsp[i]->file,dsp[i]->bits_per_sample,
			 dsp[i]->channels,dsp[i]->sample_rate) == -1)
	      {
		warning("Init.c: mixer_initialize: Error while configuring "
			"the DSP - return (-3)");
		return(-3);
	      } /* fi */
	} /* fi */
      
      mixer_elt = mixer_elt->next_mixer_elt;

    } /* While */

  return(0);

} /* mixer_initialize */

int mixer_seek_start(main_mixer)
     t_main_mixer* main_mixer;
{
  t_mixer_elt* mixer_elt;
  
  mixer_elt = main_mixer->first_mixer_elt;
  main_mixer->played_samples = 0;

  while (mixer_elt != NULL)
  {
    lseek(mixer_elt->tempgmurf->file, 0, SEEK_SET);
    mixer_elt = mixer_elt->next_mixer_elt;
  }
  return (0);
  
} // mixer_seek_start

/*
** ---------- mixer_release
**
** Unlocks all an  re-open all the DSPs in write mode.
*/

int mixer_release(main_mixer)
     t_main_mixer* main_mixer;
{
  t_mixer_elt* mixer_elt;

  mixer_elt = main_mixer->first_mixer_elt;

  while (mixer_elt != NULL)
    {
 
      if (mixer_elt->dsp->rec_or_play == REC)
	{
	  dsp_close(mixer_elt->dsp->file);
	  if ((mixer_elt->dsp->file = open(mixer_elt->dsp->name,O_WRONLY)) 
	      == -1 ) 
	    {
	      warning("Init.c: mixer_release: Erreur lors de "
		      "l'ouverture du device DSP en criture");
	    } /* fi */
	  
	  mixer_elt->dsp->rec_or_play = PLAY;
	  
	} /* fi */
  
      mixer_elt->dsp->lock = FALSE_;
      // reread size
      mixer_elt->tempgmurf->temp_size = 
	give_file_size(mixer_elt->tempgmurf->file);
      mixer_elt = mixer_elt->next_mixer_elt;
    } /* while */

  return(0);
  
} /* mixer_release */

